/*------------------------------------------------------------------------------
 bin/djv_view/playback.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_PLAYBACK_H
#define DJV_VIEW_PLAYBACK_H

#include <djv_ui/num.h>
#include <djv_ui/slider.h>
#include <djv_ui/choice.h>
#include <djv_ui/num_float.h>
#include "base.h"

namespace playback {

// Playback.

enum PLAYBACK { STOP, FORWARD, REVERSE };
enum FRAME { START, END, PREV, PREV_10, PREV_100, NEXT, NEXT_10, NEXT_100 };

// Frame widget.

class Frame_Widget : public ui::widget::Widget, public ui::num::Int_Base {
public:

  Frame_Widget();
  
  // Value.
  
  void speed(const image::time::Speed &);
  const image::time::Speed & speed() const { return _speed; }
  
  // Widget.
  
  void text_font(const Fl_Font &);
  void text_color(const Fl_Color &);
  const Fl_Font & text_font() const { return Widget::text_font(); }
  const Fl_Color & text_color() const { return Widget::text_color(); }
  
  int handle(int);
  
  // Layout.
  
  void dirty();
  ui::layout::Item * group_align();
  
private:

  DJV_CALLBACK(Frame_Widget, widget_callback, const Str &);
  DJV_CALLBACK(Frame_Widget, inc_callback, bool);
  DJV_CALLBACK(Frame_Widget, dec_callback, bool);

  void value_update();

  int _frame;
  image::time::Speed _speed;
  ui::text::Edit * _widget;
};

// Frame slider widget.

class Frame_Slider : public ui::slider::Int {
public:

  Frame_Slider();
  
  // Value.

  void in_out(int, int);
  int in() const { return _in; }
  int out() const { return _out; }
  
  // Widget.
  
  void draw();
  
private:

  int _in, _out;
};

// Time widget.

class Time_Widget : public ui::widget::Widget {
public:

  Time_Widget();
  
  // Value.

  void frame(int);
  void speed(const image::time::Speed &);
  void mark(bool);
  void range(int, int);
  int frame() const { return _frame; }
  const image::time::Speed & speed() const { return _speed; }
  bool mark() const { return _mark; }
  ui::Signal<int> mark_sig;
  
  // Layout.
  
  void dirty();
  
private:

  DJV_CALLBACK(Time_Widget, widget_callback, int);
  
  void value_update();
  void widget_update();
  
  int _frame;
  image::time::Speed _speed;
  bool _mark;
  ui::text::Display * _widget;
  Frame_Widget * _mark_widget;
};

// Speed widget.

class Speed_Widget : public ui::widget::Widget {
public:

  Speed_Widget();
  
  // Value.
  
  void set(const image::time::Speed &);
  void set_default(const image::time::Speed &);
  ui::Signal<const image::time::Speed &> sig;
  
  // Layout.
  
  void dirty();

private:

  DJV_CALLBACK(Speed_Widget, menu_callback, int);
  
  void widget_update();
  void menu_update();

  image::time::Speed _value, _value_default;
  ui::text::Display * _widget;
  ui::menu::Popup * _menu;
};

// Speed display widget.

class Speed_Display_Widget : public ui::widget::Widget {
public:

  Speed_Display_Widget();
  
  // Value.
  
  void set(double);
  void dropped_frames(bool);
  
  // Layout.
  
  void dirty();
  
private:

  void widget_update();

  double _value;
  bool _dropped_frames;
  
  ui::text::Display * _widget;
};

}

#endif

