/*------------------------------------------------------------------------------
 bin/djv_view/window_image.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "window.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/button.h>
#include <djv_ui/menu.h>
#include <djv_ui/app.h>
#include "prefs.h"
#include "image_color.h"
#include "magnify.h"
#include "color_sample.h"
#include "histogram.h"
#include "image_info.h"

namespace window {

Image_Group::Image_Group() : _menu(0) {}

void Image_Group::menu(ui::menu::Widget * in, bool mirror_x, bool mirror_y,
  ui::image::SCALE scale, ui::image::ROTATE rotate, ui::image::CHANNEL channel,
  ui::image::FILTER filter
) {
  List<Str> tmp;
  
  in->menu_add("I&mage", 0, 0, 0, ui::menu::SUB_MENU);

  in->menu_add("Mirror Hori&zontal", FL_F + 1, _mirror_x_callback,
    this, ui::menu::TOGGLE | (mirror_x ? ui::menu::VALUE : 0));
  in->menu_add("Mirror &Vertical", FL_F + 2, _mirror_y_callback, this,
    ui::menu::TOGGLE | (mirror_y ? ui::menu::VALUE : 0));
  
  in->menu_add("Sc&ale", 0, 0, 0, ui::menu::SUB_MENU);
  _menu_scale.clear();
  tmp = ui::image::scale_label();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_scale += in->menu_add(
      tmp[i], FL_F + 5 + i, _scale_callback, this,
      ui::menu::RADIO | (int(i) == scale ? ui::menu::VALUE : 0));
  in->menu_end();
  
  in->menu_add("&Rotate", 0, 0, 0, ui::menu::SUB_MENU | ui::menu::DIVIDER);
  _menu_rotate.clear();
  tmp = ui::image::rotate_label();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_rotate += in->menu_add(
      tmp[i], FL_F + 9 + i, _rotate_callback, this,
      ui::menu::RADIO | (int(i) == rotate ? ui::menu::VALUE : 0));
  in->menu_end();

  in->menu_add("&Color", 0, 0, 0, ui::menu::SUB_MENU);
  in->menu_add("&Show", 'c', _color_callback, this);
  in->menu_add("&Reset", FL_ALT + '`', _color_reset_callback,
    this, ui::menu::DIVIDER);
  in->menu_add("&Add", 0, _color_add_callback, this);
  in->menu_add("&Delete", 0, _color_del_callback, this);
  in->menu_add("Delete A&ll", 0, _color_del_all_callback, this,
    ui::menu::DIVIDER);
  _menu_color.clear();
  tmp = prefs::image()->color_name_list();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_color += in->menu_add(
      ui::menu::list_name(tmp[i], i), i < 9 ? (FL_ALT + '1' + i) : 0,
      _color_list_callback, this);
  in->menu_end();
  
  in->menu_add("C&hannel", 0, 0, 0, ui::menu::SUB_MENU | ui::menu::DIVIDER);
  _menu_channel.clear();
  const int channel_shortcut [] = { '`', '1', '2', '3', '4' };
  tmp = ui::image::channel_label();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_channel += in->menu_add(
      tmp[i], channel_shortcut[i], _channel_callback, this,
      ui::menu::RADIO | (int(i) == channel ? ui::menu::VALUE : 0));
  in->menu_end();
  
  in->menu_add("&Filter", 0, 0, 0, ui::menu::SUB_MENU | ui::menu::DIVIDER);
  _menu_filter.clear();
  tmp = ui::image::filter_label();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_filter += in->menu_add(
      tmp[i], 0, _filter_callback, this,
      ui::menu::RADIO | (int(i) == filter ? ui::menu::VALUE : 0));
  in->menu_end();
  
  in->menu_add("&Magnify", 'm', _magnify_callback, this);
  in->menu_add("Color &Sample", 's', _color_sample_callback, this);
  in->menu_add("Histo&gram", 'g', _histogram_callback, this);
  in->menu_add("I&nformation", 'n', _info_callback, this);
  
  in->menu_end();
  
  _menu = in;
}

void Image_Group::toolbar(ui::layout::Horizontal * in)
{
  ui::button::Tool * color_widget = new ui::button::Tool("image_color");
  color_widget->tooltip("Show color dialog. (Shortcut: C)");

  ui::button::Tool * magnify_widget = new ui::button::Tool("magnify");
  magnify_widget->tooltip("Show magnify dialog. (Shortcut: M)");

  ui::button::Tool * color_sample_widget = new ui::button::Tool("color_sample");
  color_sample_widget->tooltip("Show color sample dialog. (Shortcut: S)");
  
  in->add(color_widget);
  in->add(magnify_widget);
  in->add(color_sample_widget);

  color_widget->sig.set(this, color_callback);
  magnify_widget->sig.set(this, magnify_callback);
  color_sample_widget->sig.set(this, color_sample_callback);
}

void Image_Group::_mirror_x_callback() { mirror_x_sig.emit(_menu->value()); }
void Image_Group::_mirror_y_callback() { mirror_y_sig.emit(_menu->value()); }

void Image_Group::_scale_callback()
  { scale_sig.emit(ui::image::SCALE(list::find(_menu->menu(), _menu_scale))); }

void Image_Group::_rotate_callback()
  { rotate_sig.emit(ui::image::ROTATE(
      list::find(_menu->menu(), _menu_rotate))); }

void Image_Group::color_callback(bool) { color_sig.emit(true); }
void Image_Group::_color_callback() { color_callback(true); }

void Image_Group::_color_reset_callback() { color_reset_sig.emit(true); }

void Image_Group::color_add_callback(const Str & in) { color_add_sig.emit(in); }

void Image_Group::_color_add_callback()
{
  ui::dialog::input("Add color setting:", "Color", this, color_add_callback);
}

void Image_Group::color_del_callback(int in)
{
  List<Color> list = prefs::image()->color_list();
  List<Color>::iterator i = list.begin();
  for (int j = 0; j < in && i != list.end(); ++j, ++i) ;
  list.erase(i);
  prefs::image()->color_list(list);
}

void Image_Group::_color_del_callback()
{
  const List<Str> list = prefs::image()->color_name_list();
  if (! list.size()) return;
  ui::dialog::choice("Delete color setting:", list, this, color_del_callback);
}

void Image_Group::color_del_all_callback(bool)
  { prefs::image()->color_list(List<Color>()); }

void Image_Group::_color_del_all_callback()
{
  const List<Str> list = prefs::image()->color_name_list();
  if (! list.size()) return;
  ui::dialog::question("Delete all color settings?", this,
    color_del_all_callback);
}

void Image_Group::_color_list_callback()
  { color_value_sig.emit(prefs::image()->color_list()[
      list::find(_menu->menu(), _menu_color)]); }

void Image_Group::_channel_callback()
  { channel_sig.emit(ui::image::CHANNEL(
      list::find(_menu->menu(), _menu_channel))); }

void Image_Group::_filter_callback()
  { filter_sig.emit(ui::image::FILTER(
      list::find(_menu->menu(), _menu_filter))); }

void Image_Group::magnify_callback(bool) { magnify_sig.emit(true); }
void Image_Group::_magnify_callback() { magnify_callback(true); }

void Image_Group::color_sample_callback(bool) { color_sample_sig.emit(true); }
void Image_Group::_color_sample_callback() { color_sample_callback(true); }

void Image_Group::_histogram_callback() { histogram_sig.emit(true); }

void Image_Group::_info_callback() { info_sig.emit(true); }

void Widget::image_mirror_x_callback(bool in)
{
  _image_mirror_x = in;
  image_display_update();
  image_dialog_update();
  menu_update();
}

void Widget::image_mirror_y_callback(bool in)
{
  _image_mirror_y = in;
  image_display_update();
  image_dialog_update();
  menu_update();
}

void Widget::image_scale_callback(ui::image::SCALE in)
{
  _image_scale = in;
  image_display_update();
  image_dialog_update();
  image_resize_update();
  menu_update();
}

void Widget::image_rotate_callback(ui::image::ROTATE in)
{
  _image_rotate = in;
  image_display_update();
  image_dialog_update();
  image_resize_update();
  menu_update();
}

void Widget::image_color_callback(bool)
{
  image_color::dialog()->sig.del();
  image_color::dialog()->sig.set(this, image_color_callback);
  image_color::dialog()->set(&_image_color_sig);
  _image_color_sig.emit(_image_color);
  image_color::dialog()->show();
}

void Widget::image_color_callback(const Color & in)
{
  DJV_DBG_OBJ("window::Widget::image_color_callback");
  DJV_DBG_PRN("in = " << in);
  
  _image_color = in;
  image_display_update();
  image_dialog_update();
  _image_color_sig.emit(_image_color);
}

void Widget::image_color_reset_callback(bool)
{
  image_color_callback(Color());
}

void Widget::image_color_list_callback(bool)
{
  menu_update();
}

void Widget::image_color_add_callback(const Str & in)
{
  List<Color> list = prefs::image()->color_list();
  Color tmp = _image_color;
  tmp.name = in;
  list += tmp;
  prefs::image()->color_list(list);
}

void Widget::image_channel_callback(ui::image::CHANNEL in)
{
  _image_channel = in;
  image_display_update();
  image_dialog_update();
  menu_update();
}

void Widget::image_filter_callback(ui::image::FILTER in)
{
  _image_filter = in;
  image_display_update();
  image_dialog_update();
  menu_update();
}

void Widget::image_magnify_callback(bool)
{
  magnify::dialog()->set(_view, _image_pick);
  magnify::dialog()->show();
}

void Widget::image_color_sample_callback(bool)
{
  color_sample::dialog()->set(_view, _image_pick);
  color_sample::dialog()->show();
}

void Widget::image_histogram_callback(bool)
{
  histogram::dialog()->set(_view);
  histogram::dialog()->show();
}

void Widget::image_info_callback(bool)
{
  image_info::dialog()->set(this, _info);
  image_info::dialog()->show();
}

void Widget::image_update()
{
  DJV_DBG_OBJ("window::Widget::image_update");
  DJV_DBG_PRN("frame = " << _playback_frame);
  
  callbacks(false);

  // Get image.

  _image_p = 0;
  
  cache::Ref * prev_ref = _image_ref;
  _image_ref = _file_cache ?
    cache::cache()->get(int(this), _playback_frame) : 0;

  if (_image_ref) {

    DJV_DBG_PRN("referenced image");
    
    _image_p = _image_ref->get();
    
  } else {

    DJV_DBG_PRN("loading image");
    
    try {
      if (_load.get())
        _image_p = _load->get(_playback_frame, _file_proxy);
    }
    catch (Error error) {
      DJV_DBG_PRN("error");
    }

    if (_image_p && _file_cache) {
      _image_ref = cache::cache()->create(
        new image::Image(*_image_p), int(this), _playback_frame);
      _image_p = _image_ref->get();
    }
  }
  if (_image_p) DJV_DBG_PRN("image = " << *_image_p);

  // Initialize.
  
  _image_data.set(_image_p);
  
  view_overlay_update();
  image_display_update();
  image_dialog_update();
  
  // Release old image.

  if (prev_ref) prev_ref->ref_del();

  callbacks(true);
}

void Widget::image_display_update()
{
  DJV_DBG_OBJ("window::Widget::image_display_update");
  
  callbacks(false);
  
  // XXX
  
  if (! GLEW_ARB_fragment_shader)
    _image_data.set(_image_p);

  _view->mirror(_image_mirror_x, _image_mirror_y);
  _view->scale(_image_scale);
  _view->rotate(_image_rotate);
  _view->profile(_image_p ? _image_p->info().profile :
    image::profile::Profile());
  _view->display(_image_color);
  _view->channel(_image_channel);
  _view->filter(_image_filter);
  _view->redraw();

  callbacks(true);
}

void Widget::image_resize_update()
{
  if (prefs::window()->resize()) {
    if (! fullscreen()) window_fit();
    else _view->view_fit();
  }
}

void Widget::image_dialog_update()
{
  callbacks(false);
  
  magnify::dialog()->update(_view, _image_pick);
  color_sample::dialog()->update(_view, _image_pick);
  histogram::dialog()->update(_view);
  image_info::dialog()->update(this, _info);
  
  callbacks(true);
}

void Widget::image_pick_update()
{
  DJV_DBG_OBJ("window::Widget::image_pick_update");
  
  callbacks(false);
  
  magnify::dialog()->set(_view, _image_pick);
  color_sample::dialog()->set(_view, _image_pick);
  histogram::dialog()->set(_view);
  image_info::dialog()->set(this, _info);

  magnify::dialog()->update(_view, _image_pick);
  color_sample::dialog()->update(_view, _image_pick);
  
  callbacks(true);
}

}

