/*------------------------------------------------------------------------------
 lib/djv_base/dbg.h
 
 Debugging

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_DBG_H
#define DJV_DBG_H

#include <djv_base/list.h>
#include <djv_base/time.h>

namespace djv {
namespace dbg {

// Debugging assertion.

#if defined(DJV_ASSERT)
#undef DJV_ASSERT
#define DJV_ASSERT(IN) if (! (IN)) djv::dbg::_djv_assert(__FILE__, __LINE__)
#else
#define DJV_ASSERT(IN)
#endif

void _djv_assert(const char * file, int line);

// Debugging messages.

class Dbg {
public:

  Dbg(const Str &);
  Dbg(const Str & prefix, const Str &);
  ~Dbg();

  enum LINE { BEGIN, END };
  Dbg & operator << (LINE);

  Dbg & operator << (const Str &);
  Dbg & operator << (const char *);

private:

  void _Dbg(const Str &);
  void add(const Str &);
  
  Str _tmp;
  time::Timer _timer;
};

Dbg & operator << (Dbg &, bool);
Dbg & operator << (Dbg &, int);
Dbg & operator << (Dbg &, uint);
Dbg & operator << (Dbg &, double);
Dbg & operator << (Dbg &, const List<Str> &);
template<class T>
inline Dbg & operator << (Dbg &, const List<T> &);

#if defined(DJV_DBG)
#define DJV_DBG_OBJ(in) djv::dbg::Dbg _dbg(in)
#define DJV_DBG_OBJ2(prefix, in) djv::dbg::Dbg _dbg(prefix, in)
#define DJV_DBG_PRN(in) \
  _dbg << djv::dbg::Dbg::BEGIN << in << djv::dbg::Dbg::END
#else
#define DJV_DBG_OBJ(in)
#define DJV_DBG_OBJ2(prefix, in)
#define DJV_DBG_PRN(in)
#endif

// Conversion.

Str u8(uint8_t);
Str u16(uint16_t);
Str u32(uint32_t);

}
}

#include <djv_base/dbg_.h>

#endif

