/*------------------------------------------------------------------------------
 lib/djv_base/str_format.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/str_format.cxx"

#include "str.h"
#include <djv_base/dbg.h>

namespace djv {
namespace str {

Format Format::arg(const Str & in, int width, char fill) const
{
  //DJV_DBG_OBJ2(_DBG, "Format::arg");
  //DJV_DBG_PRN("value = " << _value);
  //DJV_DBG_PRN("in = " << in << " (" << in.size() << ")");
  //DJV_DBG_PRN("width = " << width);
  //DJV_DBG_PRN("fill = " << fill);
  
  // Find lowest occurrence of "%#".

  int size = int(_value.size()), min = -1, index = -1, digits = -1;
  for (int i = 1, j = 1; i < size; ++i, ++j) {
    if ('%' == _value[i - 1]) {
      const int start = j;
      for (; is_digit(_value[i]) && i < size; ++i, ++j) ;
      if (j != start) {
        const int size = j - start;
        int tmp = str_to_int(Str(_value, start, size));
        if (-1 == min || tmp < min) {
          min = tmp;
          index = start - 1;
          digits = size;
          //DJV_DBG_PRN("min = " << min);
        }
      }
    }
  }
  if (-1 == index) return _value;
  //DJV_DBG_PRN("index = " << index);
  //DJV_DBG_PRN("digits = " << digits);
  
  // Format input.

  Str tmp = in;
  if (width) {
    int w = (width < 0 ? -width : width) - int(in.size());
    if (w < 0) w = 0;
    if (width < 0) tmp += Str(w, fill);
    else tmp.insert(tmp.begin(), w, fill);
  }
  
  // Replace.
  
  const Str out = Str(_value, 0, index) + tmp +
    Str(_value, index + digits + 1, _value.size() - (index + digits + 1));
  //DJV_DBG_PRN("out = " << out);
  return out;
}

Format Format::arg(int in, int width, char fill) const
  { return arg(int_to_str(in), width, fill); }
Format Format::arg(uint in, int width, char fill) const
  { return arg(int_to_str(in), width, fill); }
Format Format::arg(double in, int width, int precision, char fill) const
  { return arg(float_to_str(in, precision), width, fill); }

}
}
