/*------------------------------------------------------------------------------
 lib/djv_image/gl_data_fixed.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/gl_data_fixed.cxx"

#include "_gl_data.h"
#include <djv_base/mtx.h>

namespace djv {
namespace image {
namespace gl {
namespace data {

namespace {

const Str
  error_namespace = "OpenGL";

}

void Data::_Fixed::draw(const Options & in) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Data::_Fixed::draw");

  // Profile.
  
  if (profile::LUT == in.profile.type &&
    vec::size_valid(in.profile.lut.size())
  ) {
    GLint max = 0;
    DJV_DBG_GL(glGetIntegerv(GL_MAX_PIXEL_MAP_TABLE, &max));
    DJV_DBG_PRN("pixel map max = " << max);

    const int lut_size = math::min(in.profile.lut.size().x, int(max));
    DJV_DBG_PRN("profile lut size = " << lut_size);
    
    image::Data tmp(image::data::Info(Vec2i(lut_size, 1), pixel::RGBA_F32));
    Options options;
    options.scale = tmp.info().size;
    data::copy(in.profile.lut, &tmp, options);

    mem::Buffer<GLfloat> lut_data(lut_size * 4);
    const pixel::F32_T * in_p = reinterpret_cast<pixel::F32_T *>(tmp.data());
    GLfloat * out_p = lut_data();
    for (int i = 0; i < lut_size; ++i, in_p += 4, out_p += 1) {
      //DJV_DBG_PRN("lut[" << i << "] = " << in_p[0]);
      out_p[lut_size * 0] = in_p[0];
      out_p[lut_size * 1] = in_p[1];
      out_p[lut_size * 2] = in_p[2];
      out_p[lut_size * 3] = in_p[3];
    }

    DJV_DBG_GL(glPixelTransferi(GL_MAP_COLOR, GL_TRUE));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_R_TO_R,
      lut_size, lut_data() + lut_size * 0));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_G_TO_G,
      lut_size, lut_data() + lut_size * 1));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_B_TO_B,
      lut_size, lut_data() + lut_size * 2));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_A_TO_A,
      lut_size, lut_data() + lut_size * 3));
  }

  // Display.
  
  if (GLEW_ARB_imaging && vec::size_valid(in.display.lut.size())) {

    image::Data tmp(image::data::Info(
      Vec2i(math::to_pow2(in.display.lut.size().x), 1), pixel::RGBA_U8));
    Options options;
    options.scale = tmp.info().size;
    data::copy(in.display.lut, &tmp, options);

    DJV_DBG_GL(glEnable(GL_COLOR_TABLE));
    DJV_DBG_GL(glColorTable(GL_COLOR_TABLE, GL_RGBA, tmp.w(),
      GL_RGBA, GL_UNSIGNED_BYTE, tmp.data()));
  }
  
  DJV_DBG_GL(glMatrixMode(GL_COLOR));
  DJV_DBG_GL(glLoadMatrixf(color_mtx(in.display.color).e));
  DJV_DBG_GL(glMatrixMode(GL_MODELVIEW));

  if (GLEW_ARB_imaging && (in.display.levels != Display().levels ||
    in.display.soft_clip != Display().soft_clip)
  ) {
    const image::Data lut = color_levels(in.display.levels,
      in.display.soft_clip);
    
    image::Data tmp(image::data::Info(
      Vec2i(math::to_pow2(lut.size().x), 1), pixel::RGBA_U8));
    Options options;
    options.scale = tmp.info().size;
    data::copy(lut, &tmp, options);

    DJV_DBG_GL(glEnable(GL_POST_COLOR_MATRIX_COLOR_TABLE));
    DJV_DBG_GL(glColorTable(GL_POST_COLOR_MATRIX_COLOR_TABLE, GL_RGBA,
      tmp.w(), GL_RGBA, GL_UNSIGNED_BYTE, tmp.data()));
  }
  
  if (in.channel > 0) {
    bool mask [] = { GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE };
    mask[int(in.channel) - 1] = GL_TRUE;
    DJV_DBG_GL(glColorMask(mask[0], mask[1], mask[2], mask[3]));
  }
}

}
}
}
}

