/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon.h

 Kodak Cineon Image I/O
 
 References:
 * Kodak, "4.5 DRAFT - Image File Format Proposal for Digital Pictures"

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_CINEON_H
#define DJV_IMAGE_IO_CINEON_H

#include <djv_image/io_dpx.h>

namespace djv {
namespace image {
namespace io {
namespace cineon {

// Information.

static const Str plugin_name = "Cineon";
static const List<Str> extension_list = List<Str>() << ".cin";

enum TAG { TAG_SOURCE_OFFSET, TAG_SOURCE_FILE, TAG_SOURCE_TIME,
  TAG_SOURCE_INPUT_DEVICE, TAG_SOURCE_INPUT_MODEL, TAG_SOURCE_INPUT_SERIAL,
  TAG_SOURCE_INPUT_PITCH, TAG_SOURCE_GAMMA, TAG_FILM_FORMAT, TAG_FILM_FRAME,
  TAG_FILM_FRAME_RATE, TAG_FILM_FRAME_ID, TAG_FILM_SLATE };

// Load.

class Load : public io::Load {
public:

  Load() : _print(false) {}

  Str name() const { return cineon::plugin_name; }
  const List<Str> & extension_list() const { return cineon::extension_list; }
  Str help() const;
  
  Plugin * copy() const;

  struct Options {
    Options() : color(dpx::COLOR_AUTO) {}
    dpx::COLOR color;
    dpx::Unprint print;
  };
  enum OPTION { OPTION_COLOR, OPTION_PRINT };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, Info *) throw (Error);
  const Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);

  Options _options;
  File _file;
  bool _print;
  std::auto_ptr<Data> _print_lut;
  Cache _cache;
  djv::Io _io;
  Image _image, _image_tmp;
};

// Save.

class Save : public io::Save {
public:
  
  Str name() const { return cineon::plugin_name; }
  const List<Str> & extension_list() const { return cineon::extension_list; }
  Str help() const;
  
  Plugin * copy() const;

  struct Options {
    Options() : color(dpx::COLOR_PRINT) {}
    dpx::COLOR color;
    dpx::Print print;
  };
  enum OPTION { OPTION_COLOR, OPTION_PRINT };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const Info &) throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &, const tag::Tag &) throw (Error);

  Options _options;
  File _file;
  Info _info;
  djv::Io _io;
  Image _image;
};

// Functions.

void header_set(djv::Io *, const Str & file, const image::Info &, dpx::COLOR)
  throw (Error);
void header_update(djv::Io *) throw (Error);
void header_get(djv::Io *, Info *, bool * print = 0) throw (Error);

// Labels.

const List<Str> & load_option_label();
const List<Str> & save_option_label();

}
}
}
}

#endif
