/*------------------------------------------------------------------------------
 lib/djv_image/io_sgi.h

 Silicon Graphics (SGI) Image I/O
 
 References:
 * Paul Haeberli, "The SGI Image File Format, Version 1.00"

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_SGI_H
#define DJV_IMAGE_IO_SGI_H

#include <djv_image/io.h>

namespace djv {
namespace image {
namespace io {
namespace _sgi { // XXX IRIX defines "sgi".

// Information.

static const Str plugin_name = "SGI";
static const List<Str> extension_list = List<Str>() << ".sgi" << ".rgba" <<
  ".rgb" << ".bw";

// Options.

enum COMPRESS { COMPRESS_NONE, COMPRESS_RLE };

// Load.

class Load : public io::Load {
public:

  Load();

  Str name() const { return _sgi::plugin_name; }
  const List<Str> & extension_list() const { return _sgi::extension_list; }
  
  Plugin * copy() const { return new Load; }

  void open(const File &, io::Info *) throw (Error);
  const Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, io::Info *) throw (Error);

  File _file;
  djv::Io _io;
  bool _compress;
  mem::Buffer<uint32_t> _rle_offset, _rle_size;
  Image _image, _image_tmp;
};

// Save.

class Save : public io::Save {
public:

  Save();
  
  Str name() const { return _sgi::plugin_name; }
  const List<Str> & extension_list() const { return _sgi::extension_list; }
  Str help() const;

  Plugin * copy() const;

  struct Options {
    Options() : compress(COMPRESS_NONE) {}
    COMPRESS compress;
  };
  enum OPTION { OPTION_COMPRESS };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const io::Info &) throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &) throw (Error);

  Options _options;
  File _file;
  djv::Io _io;
  mem::Buffer<uint32_t> _rle_offset, _rle_size;
  Image _image, _image_tmp;
};

// Functions.

void rle_unpack(djv::Io *, void *, int size, int bytes) throw (Error);
size_t rle_pack(const void *, void *, int size, int bytes);

// Labels.

const List<Str> & compress_label();
const List<Str> & save_option_label();

}
}
}
}

#endif
