/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_builtin.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/image_io_builtin.cxx"

#include "image_io_builtin.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/choice.h>
#include <djv_ui/num_int.h>
#include <djv_ui/num_float.h>

namespace djv {
namespace ui {
namespace image_io {
using namespace djv::image::io;

Tga_Save::Tga_Save() :
  _plugin(0)
{
  DJV_DBG_OBJ2(_DBG, "Tga_Save::Tga_Save");
  
  // Widgets.
  
  group::Box * compress_group = new group::Box;
  _compress_widget = new choice::Widget(tga::compress_label());
  _compress_widget->label(new label::Widget("Compress:"));
  compress_group->label_widget(_compress_widget);
  ui::label::Text * compress_text = new ui::label::Text("File compression.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(compress_group);
  layout::Vertical * group_layout = new layout::Vertical(compress_group);
  group_layout->add(compress_text);
  
  layout->add_stretch();
  
  // Initialize.

  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _compress_widget->sig.set(this, compress_callback);
}

Tga_Save::~Tga_Save()
{
  DJV_DBG_OBJ2(_DBG, "Tga_Save::~Tga_Save");
}

void Tga_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Tga_Save::compress_callback(int in)
{
  if (! _plugin) return;
  _options.compress = tga::COMPRESS(in);
  plugin_update();
  sig.emit(_plugin);
}

void Tga_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.compress, tga::compress_label());
    _plugin->option(_plugin->option_list()[tga::Save::OPTION_COMPRESS], &tmp);
  }
  
  callbacks(true);
}

void Tga_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[tga::Save::OPTION_COMPRESS]);
    str::serialize(&tmp, &_options.compress, tga::compress_label());
  }
  
  _compress_widget->set(_options.compress);
  
  callbacks(true);
}

Sgi_Save::Sgi_Save() :
  _plugin(0)
{
  // Widgets.
  
  group::Box * compress_group = new group::Box;
  _compress_widget = new choice::Widget(_sgi::compress_label());
  _compress_widget->label(new label::Widget("Compress:"));
  compress_group->label_widget(_compress_widget);
  ui::label::Text * compress_text = new ui::label::Text("File compression.");
  
  // Layout.
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(compress_group);
  layout::Vertical * group_layout = new layout::Vertical(compress_group);
  group_layout->add(compress_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _compress_widget->sig.set(this, compress_callback);
}

void Sgi_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Sgi_Save::compress_callback(int in)
{
  if (! _plugin) return;
  _options.compress = _sgi::COMPRESS(in);
  plugin_update();
  sig.emit(_plugin);
}

void Sgi_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.compress, _sgi::compress_label());
    _plugin->option(_plugin->option_list()[_sgi::Save::OPTION_COMPRESS], &tmp);
  }
  
  callbacks(true);
}

void Sgi_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[_sgi::Save::OPTION_COMPRESS]);
    str::serialize(&tmp, &_options.compress, _sgi::compress_label());
  }
  
  _compress_widget->set(_options.compress);
  
  callbacks(true);
}

Ppm_Save::Ppm_Save() :
  _plugin(0)
{
  // Widgets.
  
  group::Box * type_group = new group::Box;
  _type_widget = new choice::Widget(ppm::type_label());
  _type_widget->label(new label::Widget("Type:"));
  type_group->label_widget(_type_widget);
  ui::label::Text * type_text = new ui::label::Text("Pixel type.");
  
  group::Box * mode_group = new group::Box;
  _mode_widget = new choice::Widget(ppm::mode_label());
  _mode_widget->label(new label::Widget("Mode:"));
  mode_group->label_widget(_mode_widget);
  ui::label::Text * mode_text = new ui::label::Text("File mode.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(type_group);
  layout::Vertical * group_layout = new layout::Vertical(type_group);
  group_layout->add(type_text);
  
  layout->add(mode_group);
  group_layout = new layout::Vertical(mode_group);
  group_layout->add(mode_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _type_widget->sig.set(this, type_callback);
  _mode_widget->sig.set(this, mode_callback);
}

void Ppm_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Ppm_Save::type_callback(int in)
{
  if (! _plugin) return;
  _options.type = ppm::TYPE(in);
  plugin_update();
  sig.emit(_plugin);
}

void Ppm_Save::mode_callback(int in)
{
  if (! _plugin) return;
  _options.mode = ppm::MODE(in);
  plugin_update();
  sig.emit(_plugin);
}

void Ppm_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.type, ppm::type_label());
    _plugin->option(_plugin->option_list()[ppm::Save::OPTION_TYPE], &tmp);
    tmp = str::serialize(_options.mode, ppm::mode_label());
    _plugin->option(_plugin->option_list()[ppm::Save::OPTION_MODE], &tmp);
  }
  
  callbacks(true);
}

void Ppm_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[ppm::Save::OPTION_TYPE]);
    str::serialize(&tmp, &_options.type, ppm::type_label());
    tmp = _plugin->option(_plugin->option_list()[ppm::Save::OPTION_MODE]);
    str::serialize(&tmp, &_options.mode, ppm::mode_label());
  }
  
  _type_widget->set(_options.type);
  _mode_widget->set(_options.mode);
  
  callbacks(true);
}

Lut_Load::Lut_Load() :
  _plugin(0)
{
  // Widgets.
  
  group::Box * spec_group = new group::Box;
  _spec_widget = new choice::Widget(lut::spec_label());
  _spec_widget->label(new label::Widget("Spec:"));
  spec_group->label_widget(_spec_widget);
  ui::label::Text * spec_text = new ui::label::Text("File specification.");
  
  group::Box * type_group = new group::Box;
  _type_widget = new choice::Widget(lut::type_label());
  _type_widget->label(new label::Widget("Type:"));
  type_group->label_widget(_type_widget);
  ui::label::Text * type_text = new ui::label::Text("Pixel type.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(spec_group);
  layout::Vertical * group_layout = new layout::Vertical(spec_group);
  group_layout->add(spec_text);
  
  layout->add(type_group);
  group_layout = new layout::Vertical(type_group);
  group_layout->add(type_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _spec_widget->sig.set(this, spec_callback);
  _type_widget->sig.set(this, type_callback);
}

void Lut_Load::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Lut_Load::spec_callback(int in)
{
  if (! _plugin) return;
  _options.spec = lut::SPEC(in);
  plugin_update();
  sig.emit(_plugin);
}

void Lut_Load::type_callback(int in)
{
  if (! _plugin) return;
  _options.type = lut::TYPE(in);
  plugin_update();
  sig.emit(_plugin);
}

void Lut_Load::plugin_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.spec, lut::spec_label());
    _plugin->option(_plugin->option_list()[lut::Load::OPTION_SPEC], &tmp);
    tmp = str::serialize(_options.type, lut::type_label());
    _plugin->option(_plugin->option_list()[lut::Load::OPTION_TYPE], &tmp);
  }
  
  callbacks(true);
}

void Lut_Load::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[lut::Load::OPTION_SPEC]);
    str::serialize(&tmp, &_options.spec, lut::spec_label());
    tmp = _plugin->option(_plugin->option_list()[lut::Load::OPTION_TYPE]);
    str::serialize(&tmp, &_options.type, lut::type_label());
  }
  
  _spec_widget->set(_options.spec);
  _type_widget->set(_options.type);
  
  callbacks(true);
}

Lut_Save::Lut_Save() :
  _plugin(0)
{
  // Widgets.
  
  group::Box * spec_group = new group::Box;
  _spec_widget = new choice::Widget(lut::spec_label());
  _spec_widget->label(new label::Widget("Spec:"));
  spec_group->label_widget(_spec_widget);
  ui::label::Text * spec_text = new ui::label::Text("File specification.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(spec_group);
  layout::Vertical * group_layout = new layout::Vertical(spec_group);
  group_layout->add(spec_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _spec_widget->sig.set(this, spec_callback);
}

void Lut_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Lut_Save::spec_callback(int in)
{
  if (! _plugin) return;
  _options.spec = lut::SPEC(in);
  plugin_update();
  sig.emit(_plugin);
}

void Lut_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.spec, lut::spec_label());
    _plugin->option(_plugin->option_list()[lut::Save::OPTION_SPEC], &tmp);
  }
  
  callbacks(true);
}

void Lut_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[lut::Save::OPTION_SPEC]);
    str::serialize(&tmp, &_options.spec, lut::spec_label());
  }
  
  _spec_widget->set(_options.spec);
  
  callbacks(true);
}

}
}
}
