/*------------------------------------------------------------------------------
 lib/djv_ui/label.h
 
 Label Widgets

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_LABEL_H
#define DJV_UI_LABEL_H

#include <djv_ui/widget.h>

namespace djv {
namespace ui {
namespace label {

// Label widget.

class Widget : public widget::Widget {
public:

  Widget(const Str & = Str());
  virtual ~Widget() {}
  
  // Value.

  void set(const Str &);
  const Str & get() const { return _value; }

  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual Vec2i size_hint() const { return _size_hint; }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::label::Widget"; }
  
private:

  Str _value;
  Vec2i _size_hint;
};

// Text label widget.

class Text : public widget::Widget {
public:

  Text(const Str & = Str());
  virtual ~Text() {}
  
  // Value.

  void set(const Str &);
  const Str & get() const { return _value; }

  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void resize(int, int, int, int);
  virtual Vec2i size_hint() const { return _size_hint; }
  virtual Vec2i size_min() const { return _size_min; }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::label::Text"; }
  
private:

  Str _value;
  List<Str> _word_list;
  List<int> _word_size_list;
  int _word_size_max, _space_size, _text_height;
  Vec2i _size_hint, _size_min;
};

// Image label widget.

class Image : public widget::Widget {
public:

  Image(Fl_Image * = 0);
  virtual ~Image() {}
  
  // Value.

  void set(Fl_Image *);
  Fl_Image * get() const { return _value; }
  
  // Widget.

  virtual void draw();
  
  // Layout.

  virtual Vec2i size_hint() const;
  
private:

  Fl_Image * _value;
};

}
}
}

#endif
