/*------------------------------------------------------------------------------
 lib/djv_ui/layout.h
 
 Widget Layout

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_LAYOUT_H
#define DJV_UI_LAYOUT_H

#include <djv_base/base.h>
#include <djv_base/vec.h>
#include <djv_base/box.h>

namespace djv {
namespace ui {
namespace layout {

// Layout item.

class Item {
public:

  enum TYPE { WIDGET, LAYOUT, SPACER };

  Item(TYPE);
  virtual ~Item() = 0;
  
  TYPE type() const { return _type; }

  // Layout.
  
  virtual void update();
  virtual void dirty() {}
  virtual void resize(int, int, int, int);
  virtual Box2i geom() const { return _geom; }
  virtual Vec2i size_hint() const { return Vec2i(); }
  virtual Vec2i size_min() const { return group_hint(); }
  virtual int visible() const { return true; }

  // Size policy.

  enum SIZE_POLICY { FIXED, STRETCH };
  enum ALIGN_POLICY { LEFT, CENTER, RIGHT };

  void size_policy(SIZE_POLICY, SIZE_POLICY);
  void align_policy(ALIGN_POLICY, ALIGN_POLICY);
  const SIZE_POLICY * size_policy() const { return _size_policy; }
  const ALIGN_POLICY * align_policy() const { return _align_policy; }
  
  // XXX Cheesy way to align items.

  void group(const List<Item *> &);
  const List<Item *> & group() const { return _group; }
  Vec2i group_hint() const;
  virtual Item * group_align() { return this; }
  virtual Item * label_align() const { return 0; }

  // Parent.

  virtual void parent(Item *);
  Item * parent() const { return _parent; }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::layout::Item"; }

private:

  TYPE _type;
  Box2i _geom;
  SIZE_POLICY _size_policy [2];
  ALIGN_POLICY _align_policy [2];
  List<Item *> _group;
  Item * _parent;
};

// Layout spacer.

class Spacer : public Item {
public:
  Spacer() : Item(SPACER) {}
};

// Layout.

class Layout : public Item {
public:

  Layout(Item * parent = 0);
  virtual ~Layout() = 0;

  // Items.
  
  virtual void add(Item *);
  virtual void insert(Item *, int);
  virtual Item * remove(int);
  virtual Item * remove(Item *);
  virtual void del(int);
  virtual void del(Item *);
  virtual void del();
  const List<Item *> & list() const { return _list; }

  // Layout.

  void spacing(int);
  void margin(int);
  int spacing() const;
  int margin() const;

  virtual void dirty();
  virtual int visible() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::layout::Layout"; }
  void dbg(int indent = 0);

private:

  List<Item *> _list;
  int _spacing, _margin;
};

// Functions.

void group(const List<Item *> &);

}
}
}

#endif
