/*------------------------------------------------------------------------------
 lib/djv_ui/layout_row.h
 
 Row Layouts

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_LAYOUT_ROW_H
#define DJV_UI_LAYOUT_ROW_H

#include <djv_ui/layout.h>

namespace djv {
namespace ui {
namespace layout {

// Row layout base.

class Row_Base : public Layout {
public:

  Row_Base(Item * parent = 0);
  virtual ~Row_Base() = 0;

  void add_spacer(const Vec2i & = Vec2i(-1), bool stretch = false);
  void add_stretch(bool stretch = true);
  void stretch(int);
  void stretch(const Item *);
  int stretch() const { return _stretch; }

protected:

  enum DIR { HORIZONTAL, VERTICAL };

  static void layout(
    const List<Item *> &, List<Box2i> *,
    const Vec2i & size, DIR, int spacing, int stretch);

private:

  int _stretch;
};

// Horizontal row layout.

class Horizontal : public Row_Base {
public:

  Horizontal(Item * parent = 0);
  
  void add_spacer(int = -1, bool stretch = false);
  
  void dirty();
  void resize(int, int, int, int);
  Vec2i size_hint() const { return _size_hint; }
  
  Str dbg_name() const { return "djv::ui::layout::Horizontal"; }

private:

  Box2i _geom_prev;
  Vec2i _size_hint;
  List<Box2i> _list;
};

// Vertical row layout.

class Vertical : public Row_Base {
public:

  Vertical(Item * parent = 0);
  
  void add_spacer(int = -1, bool stretch = false);
  
  void dirty();
  void resize(int, int, int, int);
  Vec2i size_hint() const { return _size_hint; }
  
  Str dbg_name() const { return "djv::ui::layout::Vertical"; }

private:

  Box2i _geom_prev;
  Vec2i _size_hint;
  List<Box2i> _list;
};

}
}
}

#endif
