/*------------------------------------------------------------------------------
 lib/djv_ui/menu.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "menu.h"
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace menu {

class Base::Item {
public:

  Item();
  Item(const Str & label, int shortcut, Fl_Callback *, void *, int flags);
  
  mem::Buffer<char> ascii;
  int shortcut;
  Fl_Callback * callback;
  void * data;
  int flags;
};

class Widget::_Widget : public Fl_Menu_Bar {
public:
  _Widget() : Fl_Menu_Bar(0, 0, 0, 0) {}
};

Base::Item::Item() :
  shortcut(0), callback(0), data(0), flags(0)
{}

Base::Item::Item(const Str & label, int shortcut, Fl_Callback * callback,
  void * data, int flags
) :
  shortcut(shortcut),
  callback(callback), data(data),
  flags(flags)
{
  const int size = label.size();
  ascii.size(size + 1);
  mem::copy(label.c_str(), ascii(), size + 1);
}

Base::Base() :
  _depth(0)
{}

Base::~Base()
{
  menu_del();
}

int Base::menu_add(const Str & label, int shortcut,
  Fl_Callback * callback, void * data, int flags
) {
  _list += new Item(label, shortcut, callback, data, flags);
  
  if (flags & SUB_MENU) ++_depth;
  
  return int(_list.size()) - 1;
}

void Base::menu_end()
{
  _list += new Item;
  
  if (_depth != 0) --_depth;
  else {

    const uint size = _list.size();
    
    mem::Buffer<Fl_Menu_Item> tmp(size + 1);
    mem::set(0, tmp(), (size + 1) * sizeof(Fl_Menu_Item));
    
    Fl_Menu_Item * p = tmp();
    for (uint i = 0; i < size; ++i) {
      p[i].text = _list[i]->ascii();
      p[i].shortcut_ = _list[i]->shortcut;
      p[i].callback_ = _list[i]->callback;
      p[i].user_data_ = _list[i]->data;
      p[i].flags =
        (_list[i]->flags & TOGGLE ? FL_MENU_TOGGLE : 0) |
        (_list[i]->flags & RADIO ? FL_MENU_RADIO : 0) |
        (_list[i]->flags & VALUE ? FL_MENU_VALUE : 0) |
        (_list[i]->flags & SUB_MENU ? FL_SUBMENU : 0) |
        (_list[i]->flags & DIVIDER ? FL_MENU_DIVIDER : 0);
      //p[i].labelfont_ = (_list[i]->flags & SUB_MENU) ?
      //  style::style()->font_bold() : 0;
    }
    
    menu(tmp());
  }
}

void Base::menu_del()
{
  for (uint i = 0; i < _list.size(); ++i)
    delete _list[i];
  _list.clear();
  
  _depth = 0;
}

Widget::Widget()
{
  edges(false, false, false, true);
  
  _widget = new _Widget;
  _widget->box(FL_FLAT_BOX);
  Frame::add(_widget);
  
  size_policy(STRETCH, FIXED);
}

void Widget::menu(Fl_Menu_Item * in)
{
  _widget->clear();
  _widget->copy(in);
}

int Widget::menu() const
{
  return _widget->value();
}

bool Widget::value() const
{
  return _widget->mvalue()->value();
}

void Widget::dirty()
{
  Frame::dirty();

  fl_font(text_font(), style::style()->font_size());

  _widget->textfont(text_font());
  _widget->textsize(style::style()->font_size());
  _widget->textcolor(text_color());
  
  _size_hint = Vec2i(0, fl_height()) + style::style()->margin_widget() * 2;
}

void Widget::resize(int x, int y, int w, int h)
{
  Frame::resize(x, y, w, h);
  _widget->resize(x, y, w, h);
}

}
}
}
