/*------------------------------------------------------------------------------
 lib/djv_ui/scroll.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/scroll.cxx"

#include "scroll.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <FL/Fl.H>
#include <FL/Fl_Box.H>

namespace djv {
namespace ui {
namespace scroll {

Widget::Widget() :
  Fl_Scroll(0, 0, 0, 0),
  _resizable(true)
{
  end();
  
  box(FL_THIN_DOWN_FRAME);
  //Fl_Scroll::type(Fl_Scroll::BOTH);
  //Fl_Scroll::scrollbar.slider(FL_THIN_UP_BOX);
  //Fl_Scroll::hscrollbar.slider(FL_THIN_UP_BOX);
  
  _dummy_widget = new Fl_Box(0, 0, 0, 0);
  _dummy_widget->box(FL_NO_BOX);
  _dummy_widget->hide();
  add(_dummy_widget);
  
  layout::Vertical * layout = new layout::Vertical(this);
  //layout->margin(0);
}

int Widget::handle(int in)
{
  switch (widget::Base::handle(in)) {
    case -1: return 0;
    case 1: return 1;
  }
  return Fl_Scroll::handle(in);
}

void Widget::draw()
{
  Fl_Scroll::draw();
  widget::Base::draw();
}

void Widget::resizable(bool in)
{
  _resizable = in;
  update();
}

void Widget::update()
{
  DJV_DBG_OBJ2(_DBG, "Widget::update");
  
  dirty();
  
  const Box2i geom = this->geom();
  const Vec2i size = layout()->size_hint();
  int x = geom.x + 1, y = geom.y + 1;
  int w = geom.w - 1 * 2, h = geom.h - 1 * 2;
  
  // XXX Determine scrollbar visibility. FLTK doesn't know because draw()
  // hasn't been called yet.

  bool scrollbar_v = false, scrollbar_h = false;
  if (size.y > h) {
    scrollbar_v = true;
    w -= Fl_Scroll::scrollbar.w();
  }
  if (size.x > w) {
    scrollbar_h = true;
    h -= Fl_Scroll::hscrollbar.h();
    if (! scrollbar_v && size.y > h) {
      scrollbar_v = true;
      w -= Fl_Scroll::scrollbar.w();
    }
  }
  
  if (_resizable) {
    w = math::max(w, size.x);
    h = math::max(h, size.y);
  }
  DJV_DBG_PRN("size = " << Vec2i(w, h));
  
  _dummy_widget->resize(x, y, w, h);
  layout()->resize(x, y, w, h);
  redraw();
}

void Widget::dirty()
{
  widget::Base::dirty();
  
  _size_hint =
    Vec2i(Fl_Scroll::scrollbar.w(), Fl_Scroll::hscrollbar.h()) +
    1 * 2;
  
  _size_min = style::style()->text_size();
}

void Widget::resize(int x, int y, int w, int h)
{
  DJV_DBG_OBJ2(_DBG, "Widget::resize");
  DJV_DBG_PRN("in = " << Box2i(x, y, w, h));
  
  Fl_Scroll::resize(x, y, w, h);
  widget::Base::resize(x, y, w, h);
  if (_resizable) update();
}

}
}
}
