/*------------------------------------------------------------------------------
 lib/djv_ui/text_display.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/text_display.cxx"

#include "text.h"
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Multiline_Output.H>
#include <FL/Fl_Text_Display.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace text {

class Display::Widget : public Fl_Output {
public:
  Widget() : Fl_Output(0, 0, 0, 0) { box(FL_FLAT_BOX); }
};

/*class Multiline_Display::Widget : public Fl_Multiline_Output {
public:
  Widget() : Fl_Multiline_Output(0, 0, 0, 0) { box(FL_FLAT_BOX); }
};*/

class Multiline_Display::Widget : public Fl_Text_Display {
public:

  Widget() : Fl_Text_Display(0, 0, 0, 0)
  {
    buffer(&_value);
    box(FL_FLAT_BOX);
  }
  
  void set(const Str & in) { _value.text(in.c_str()); }
  
private:

  Fl_Text_Buffer _value;
};

Display::Display(const Str & in) :
  _value(in)
{
  frame(FL_DOWN_FRAME);
  
  add(_widget = new Widget);

  size_policy(STRETCH, FIXED);

  widget_update();
}

void Display::set(const Str & in)
{
  if (in == _value) return;
  
  DJV_DBG_OBJ2(_DBG, "Display::set");
  DJV_DBG_PRN("in = " << in);
  
  _value = in;
  widget_update();
}

void Display::text_font(const Fl_Font & in)
{
  Frame::text_font(in);
  _widget->textfont(in);
}

void Display::text_color(const Fl_Color & in)
{
  Frame::text_color(in);
  _widget->textcolor(in);
}

void Display::dirty()
{
  Frame::dirty();
  
  _widget->textfont(text_font());
  _widget->textsize(style::style()->font_size());
  _widget->textcolor(text_color());
  
  fl_font(text_font(), style::style()->font_size());
  Vec2i size;
  if (_size_str.size())
    size = str_size(_size_str);
  else
    size = Vec2i(style::style()->button_size(), fl_height());

  const int text_margin = 2;
  _size_hint = size + text_margin * 2 + frame_size() * 2;
}

void Display::resize(int x, int y, int w, int h)
{
  Frame::resize(x, y, w, h);
  const Box2i geom = frame_geom();
  _widget->resize(geom.x, geom.y, geom.w, geom.h);
}

void Display::size_str(const Str & in)
{
  _size_str = in;
}

void Display::widget_update()
{
  _widget->value(_value.c_str());
  _widget->position(0, 0);
}

Multiline_Display::Multiline_Display(const Str & in) :
  _value(in)
{
  frame(FL_THIN_DOWN_FRAME);
  
  add(_widget = new Widget);
  
  widget_update();
}

void Multiline_Display::set(const Str & in)
{
  if (in == _value) return;
  
  DJV_DBG_OBJ2(_DBG, "Multiline_Display::set");
  DJV_DBG_PRN("in = " << in);
  
  _value = in;
  widget_update();
}

void Multiline_Display::text_font(const Fl_Font & in)
{
  Frame::text_font(in);
  _widget->textfont(in);
}

void Multiline_Display::text_color(const Fl_Color & in)
{
  Frame::text_color(in);
  _widget->textcolor(in);
}

void Multiline_Display::dirty()
{
  Frame::dirty();
  
  _widget->textfont(text_font());
  _widget->textsize(style::style()->font_size());
  _widget->textcolor(text_color());
  
  fl_font(text_font(), style::style()->font_size());
  const Vec2i size = str_size(Str(20, '0'));
  
  const int text_margin = 2;
  _size_hint = Vec2i(size.x, size.y * 8) + text_margin * 2 + frame_size() * 2;
}

void Multiline_Display::resize(int x, int y, int w, int h)
{
  DJV_DBG_OBJ2(_DBG, "Multiline_Display::resize");
  DJV_DBG_PRN("in = " << Box2i(x, y, w, h));
  
  Frame::resize(x, y, w, h);
  
  const Box2i geom = frame_geom();
  _widget->resize(geom.x, geom.y, geom.w, geom.h);
}

void Multiline_Display::widget_update()
{
  _widget->set(_value);
}

}
}
}
