/*------------------------------------------------------------------------------
 lib/djv_ui/text_edit.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/text_edit.cxx"

#include "text.h"
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Input.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace text {

class Edit::Widget : public Fl_Input {
public:

  Widget() : Fl_Input(0, 0, 0, 0)
  {
    when(FL_WHEN_CHANGED);
    box(FL_FLAT_BOX);
  }
};

Edit::Edit(const Str & in) :
  _value(in)
{
  frame(FL_DOWN_FRAME);
  
  _widget = new Widget;
  add(_widget);

  size_policy(STRETCH, FIXED);

  widget_update();

  _widget->callback(widget_callback, this);
}

void Edit::set(const Str & in)
{
  if (in == _value) return;
  
  DJV_DBG_OBJ2(_DBG, "Edit::set");
  DJV_DBG_PRN("in = " << in);
  
  _value = in;
  widget_update();
  sig.emit(_value);
}

void Edit::text_font(const Fl_Font & in)
{
  Frame::text_font(in);
  _widget->textfont(in);
}

void Edit::text_color(const Fl_Color & in)
{
  Frame::text_color(in);
  _widget->textcolor(in);
}

int Edit::handle(int in)
{
  switch (in) {
    
    case FL_KEYBOARD:
      switch (Fl::event_key()) {
        case FL_Enter:
        case FL_KP_Enter:
          sig.emit(_widget->value());
          action_sig.emit(_widget->value());
          return 1;
      }
      break;
      
    case FL_UNFOCUS:
      sig.emit(_widget->value());
      break;
  }
  return Frame::handle(in);
}

void Edit::dirty()
{
  Frame::dirty();

  fl_font(text_font(), style::style()->font_size());

  _widget->textfont(text_font());
  _widget->textsize(style::style()->font_size());
  _widget->textcolor(text_color());
  
  int size = 0;
  if (_size_str.size())
    size = str_width(_size_str);
  else
    size = style::style()->button_size();
  
  const int text_margin = 2;
  _size_hint = Vec2i(size, fl_height()) + text_margin * 2 + frame_size() * 2;
}

void Edit::resize(int x, int y, int w, int h)
{
  Frame::resize(x, y, w, h);
  const Box2i geom = frame_geom();
  _widget->resize(geom.x, geom.y, geom.w, geom.h);
}

void Edit::size_str(const Str & in)
{
  _size_str = in;
}

void Edit::widget_callback()
{
  _value = _widget->value();
  change_sig.emit(_value);
}

void Edit::widget_update()
{
  _widget->value(_value.c_str());
  _widget->position(0, 0);
}

}
}
}
