/*------------------------------------------------------------------------------
 lib/djv_ui/window.h
 
 Window

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_WINDOW_H
#define DJV_UI_WINDOW_H

#include <djv_ui/widget.h>
#include <FL/Fl_Double_Window.H>

namespace djv {
namespace ui {
namespace window {

class Widget : public Fl_Double_Window, public widget::Base {
public:

  Widget(const Str & title = Str());
  virtual ~Widget();
  
  // Title.

  void title(const Str &);
  const Str & title() const { return _title; }
  
  // Signals.

  Signal<bool> close_sig;
  
  // Widget.

  void fullscreen(bool);
  bool fullscreen() const { return _fullscreen; }
  
  virtual int handle(int);
  virtual void draw();

  virtual void widget_del();
  
  // Layout.

  virtual void update();
  virtual void size(const Vec2i &, bool resizable = true);
  virtual void resize(int, int, int, int);
  virtual Vec2i size_hint() const;
  virtual Vec2i size_min() const;
  virtual int visible() const { return Fl_Double_Window::visible(); }

  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::window::Widget"; }

private:

  DJV_FL_WIDGET_CALLBACK(Widget, close_callback);

  Str _title;
  bool _fullscreen;
  Box2i _geom_tmp;
  bool _size_hint_init;
  bool _resizable;
};

}
}
}

#endif

