/*------------------------------------------------------------------------------
 plugin/djv_exr/plugin.h

 Industrial Light & Magic OpenEXR Image I/O
 
 Requires:
 * OpenEXR - http:://www.openexr.com

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_EXR_PLUGIN_H
#define DJV_EXR_PLUGIN_H

#include <djv_image/io.h>
#include <ImfPixelType.h>
#include <ImathBox.h>

namespace Imf {
class InputFile;
class OutputFile;
}

using namespace djv;

// Information.

static const Str plugin_name = "EXR";
static const List<Str> extension_list = List<Str>() << ".exr";

// Options.

enum COLOR { COLOR_RAW, COLOR_EXPOSURE };
enum COMPRESS { COMPRESS_NONE, COMPRESS_PIZ, COMPRESS_ZIP, COMPRESS_RLE };

// Image tags.

enum TAG {
  TAG_LONGITUDE,
  TAG_LATITUDE,
  TAG_ALTITUDE,
  TAG_FOCUS,
  TAG_EXPOSURE,
  TAG_APERTURE,
  TAG_ISO_SPEED,
  TAG_CHROMATICITIES,
  TAG_WHITE_LUMINANCE,
  TAG_X_DENSITY
};

// Load.

class Load : public image::io::Load {
public:

  Load();
  ~Load();

  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  plugin::Plugin * copy() const;
  
  struct Options {
    Options() : color(COLOR_EXPOSURE) {}
    COLOR color;
    image::profile::Exposure exposure;
  };
  enum OPTION { OPTION_COLOR, OPTION_EXPOSURE };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, image::io::Info *) throw (Error);
  const image::Image * get(int frame, int proxy) throw (Error);

private:
  
  void _open(const File &, image::io::Info *) throw (Error);
  void _close();
  
  Options _options;
  File _file;
  Cache _cache;
  Imf::InputFile * _f;
  List<Str> _channel_list;
  image::Image _image, _image_tmp;
};

// Save.

class Save : public image::io::Save {
public:

  Save();
  ~Save();
  
  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  plugin::Plugin * copy() const;
  
  struct Options {
    Options() : compress(COMPRESS_NONE) {}
    COMPRESS compress;
  };
  enum OPTION { OPTION_COMPRESS };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::io::Info &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  void _open(const File &, const image::io::Info &) throw (Error);
  void _close();

  Options _options;
  File _file;
  Imf::OutputFile * _f;
  List<Str> _channel_list;
  image::Image _image;
};

// Conversion.

Box2i box(const Imath::Box2i &);

// Labels.

const List<Str> & color_label();
const List<Str> & compress_label();
const List<Str> & tag_label();
const List<Str> & load_option_label();
const List<Str> & save_option_label();

#endif

