/*------------------------------------------------------------------------------
 plugin/djv_jpeg/plugin.h

 Joint Photographic Experts Group (JPEG) Image I/O
 
 Requires:
 * libjpeg - http://www.ijg.org
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_JPEG_PLUGIN_H
#define DJV_JPEG_PLUGIN_H

#include <djv_image/io.h>
namespace libjpeg { // XXX WIN32 conflicts?
extern "C" {
#include <jpeglib.h>
}
}
#include <stdio.h>
#include <setjmp.h>

using namespace djv;

// Information.

static const Str plugin_name = "JPEG";
static const List<Str> extension_list = List<Str>() << ".jpeg" << ".jpg" <<
  ".jfif";

// Errors.

struct Jpeg_Error {
  struct libjpeg::jpeg_error_mgr pub;
  char msg [JMSG_LENGTH_MAX];
  jmp_buf jump;
};

extern "C" {
void djv_jpeg_error(libjpeg::j_common_ptr);
void djv_jpeg_warn(libjpeg::j_common_ptr, int);
}

// Load.

class Load : public image::io::Load {
public:

  Load();
  ~Load();

  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  plugin::Plugin * copy() const { return new Load; }
  
  void open(const File &, image::io::Info *) throw (Error);
  const image::Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, image::io::Info *) throw (Error);
  void _close();

  File _file;
  FILE * _f;
  libjpeg::jpeg_decompress_struct _jpeg;
  bool _jpeg_init;
  Jpeg_Error _jpeg_error;
  image::Image _image, _image_tmp;
};

// Save.

class Save : public image::io::Save {
public:

  Save();
  ~Save();
  
  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  plugin::Plugin * copy() const;
  
  struct Options {
    Options() : quality(100) {}
    int quality;
  };
  enum OPTION { OPTION_QUALITY };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const image::io::Info &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:
  
  void _open(const File &, const image::tag::Tag &) throw (Error);
  void _close();

  Options _options;
  File _file;
  FILE * _f;
  libjpeg::jpeg_compress_struct _jpeg;
  bool _jpeg_init;
  Jpeg_Error _jpeg_error;
  image::Image _image;
};

// Labels.

const List<Str> & save_option_label();

#endif

