/*------------------------------------------------------------------------------
 plugin/djv_quicktime_unix/plugin.h
 
 UNIX QuickTime Image I/O
 
 Requires:
 * libquicktime - http://libquicktime.sourceforge.net

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_QUICKTIME_UNIX_PLUGIN_H
#define DJV_QUICKTIME_UNIX_PLUGIN_H

#include <djv_image/io.h>
#include <lqt.h>
#include <colormodels.h>

using namespace djv;

// Information.

static const Str plugin_name = "QuickTime";
static const List<Str> extension_list = List<Str>() << ".qt" << ".mov"
  << ".avi";

// Load.

class Load : public image::io::Load {
public:

  Load();
  ~Load();

  plugin::Plugin * copy() const { return new Load; }
  
  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::io::Info *) throw (Error);
  const image::Image * get(int frame, int proxy) throw (Error);

private:

  void _close();
  
  File _file;
  image::io::Info _info;
  int _frame;
  quicktime_t * _f;
  image::Image _image, _image_tmp;
};

// Save.

class Save : public image::io::Save {
public:

  Save();
  
  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  plugin::Plugin * copy() const;
  
  struct Options {
    Options() : codec("jpeg") {}
    Str codec;
  };
  enum OPTION { OPTION_CODEC };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::io::Info &) throw (Error);
  void close() throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:
  
  Options _options;
  File _file;
  quicktime_t * _f;
  int _frame_duration;
  image::Image _image;
};

// Labels.

static const Str
  error_video = "No Video Tracks",
  error_codec = "Codec",
  error_colormodel = "Color Model";

List<Str> codec_label();
const List<Str> & save_option_label();

#endif

