//#define DJV_DBG

#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/tab.h>
#include <djv_ui/image_io.h>
#include <djv_ui/app.h>

using namespace djv;

class App : public ui::app::App {
public:

  App(int & argc, char ** argv);
  ~App();

private:

  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  ui::app::App("image_io", argc, argv),
  _window(0)
{
  DJV_DBG_OBJ("App::App");
  
  // Widgets.
  
  ui::tab::Widget * tab_widget = new ui::tab::Widget;

  ui::tab::Widget * load_tab_widget = new ui::tab::Widget;
  tab_widget->tab_add("Load", load_tab_widget);
  const List<image::io::Load *> & load_list = image::io::load()->list();
  for (uint i = 0; i < load_list.size(); ++i) {
    ui::image_io::Load * widget = ui::image_io::load()->get(load_list[i]);
    if (! widget) continue;
    widget->set(load_list[i]);
    load_tab_widget->tab_add(load_list[i]->name(), widget);
  }

  ui::tab::Widget * save_tab_widget = new ui::tab::Widget;
  tab_widget->tab_add("Save", save_tab_widget);
  const List<image::io::Save *> & save_list = image::io::save()->list();
  for (uint i = 0; i < save_list.size(); ++i) {
    ui::image_io::Save * widget = ui::image_io::save()->get(save_list[i]);
    if (! widget) continue;
    widget->set(save_list[i]);
    save_tab_widget->tab_add(save_list[i]->name(), widget);
  }

  _window = new ui::window::Widget(name());

  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->margin(0);
  layout->add(tab_widget);
  
  // Initialize.
  
  _window->size(Vec2i(300, 300));
  _window->show();

  // Callbacks.
  
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
}

int main(int argc, char ** argv)
{
  return App(argc, argv).exec();
}
