//#define DJV_DBG

#include <djv_base/math.h>
#include <djv_image/color.h>
#include <djv_image/data.h>
//#include <djv_image/gen.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/tab.h>
//#include <djv_ui/color.h>
#include <djv_ui/app.h>
#include <FL/Fl_Image.H>

using namespace djv;

class Tab : public ui::label::Image {
public:
  Tab();
  ~Tab();
private:
  Fl_Image * _image;
};

Tab::Tab() :
  _image(0)
{
  image::Data tmp;
  /*image::gen::gradient(
    &tmp,
    image::data::Info(512, 512, image::pixel::RGB, image::pixel::U8),
    Vec2i(), Vec2i(512) - 1,
    image::Color(math::rand(), math::rand(), math::rand()),
    image::Color(math::rand(), math::rand(), math::rand()));*/
  
  _image = ui::image_to_fl(tmp);
  set(_image);
}

Tab::~Tab()
{
  if (_image) delete _image;
}

class App : public ui::app::App {
public:

  App(int & argc, char ** argv);
  ~App();

private:

  void add();
  void del();
  
  DJV_CALLBACK(App, widget_callback, int)
    { print(str::Format("tab = %1").arg(in)); }
  DJV_CALLBACK(App, add_callback, bool) { add(); }
  DJV_CALLBACK(App, del_callback, bool) { del(); }
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  ui::tab::Widget * _widget;
  ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  ui::app::App("tab", argc, argv),
  _window(0)
{
  // Widgets.
   
  _widget = new ui::tab::Widget;

  ui::button::Tool * add_button = new ui::button::Tool("tool_add");
  add_button->type(ui::button::Tool::REPEAT);
  ui::button::Tool * del_button = new ui::button::Tool("tool_x");
  del_button->type(ui::button::Tool::REPEAT);

  ui::group::Frame * frame = new ui::group::Frame;
  frame->edges(false, false, true, false);

  _window = new ui::window::Widget(name());
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  layout->add(frame);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(frame);
  h_layout->add_stretch();
  h_layout->add(add_button);
  h_layout->add(del_button);
  
  // Initialize.
  
  add();
  add();
  
  _window->size(Vec2i(300, 300));
  _window->show();
  
  // Callbacks.
  
  _widget->sig.set(this, widget_callback);
  add_button->sig.set(this, add_callback);
  del_button->sig.set(this, del_callback);
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
}

void App::add()
{
  _widget->tab_add(str::Format("tab %1").arg(_widget->tab_size()),
    new Tab);
    //new ui::color::Widget);
}

void App::del()
{
  if (! _widget->tab_size()) return;
  _widget->tab_del(_widget->get());
}

int main(int argc, char ** argv)
{
  return App(argc, argv).exec();
}
